function LST = tbx_cfg_LST

%     ps_LST_run.m - part of the LST toolbox
%     Copyright (C) 2012  Paul Schmidt
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.


data_pve = cfg_files;
data_pve.tag  = 'data_pve';
data_pve.name = 'PVE Label';
data_pve.help = {[...
'Select PVE-labels that were estimated by vbm8.']};
data_pve.filter = 'image';
data_pve.ufilter = '.*';
data_pve.num     = [1 Inf];

data_FLAIR = cfg_files;
data_FLAIR.tag  = 'data_FLAIR';
data_FLAIR.name = 'FLAIR volume';
data_FLAIR.help = {[...
'Select bias corrected FLAIR images.']};
data_FLAIR.filter = 'image';
data_FLAIR.ufilter = '^m.*';
data_FLAIR.num     = [1 Inf];

% ------------------------------------------------------------------------

initial = cfg_entry;
initial.tag  = 'initial';
initial.name = 'Initial threshold';
initial.strtype = 'e';
initial.num = [1 20];
initial.def  = @(val) [0.3, repmat(0, 1, 19)];
initial.help = {[...
'Threshold (kappa) for the initial lesion mask. You can choose up to twenty'...
' different values.'...
''...
' See the Schmidt et al. (2012) for details and the manual on how to ' ...
'determine the optimal value for this threshold.']};

belief = cfg_menu;
belief.tag = 'belief';
belief.name = 'Lesion belief map';
belief.labels = {'GM','WM', 'CSF', 'GM+WM', 'GM+CSF', 'WM+CSF', 'All'};
belief.values = {0 1 2 3 4 5 6};
belief.def = @(val) 0;
belief.help    = {['The lesion belief map is thresholded by the initial ' ...
    'threshold in order to produce the initial lesion mask. The original ' ...
    'implementation uses the belief map of GM only, see Schmidt et al. (2012) ' ...
    'for details. Here, the user can choose different combinations of all '... 
    'lesion belief maps. However, those should be used with care, as the ' ...
    'resulting segmentations may differ a lot from the one obtained by the default option.']};

mrf = cfg_entry;
mrf.tag  = 'mrf';
mrf.name = 'MRF parameter';
mrf.strtype = 'e';
mrf.num = [1 1];
mrf.def  = @(x) 1;
mrf.help = {[...
'Parameter for the Markov Random Field.']};

maxiter = cfg_entry;
maxiter.tag  = 'maxiter';
maxiter.name = 'Maximum iterations';
maxiter.strtype = 'e';
maxiter.num = [1 1];
maxiter.def  = @(val) 50; 
maxiter.help = {[...
'Number of maximum iterations.']};

threshold      = cfg_entry;
threshold.tag = 'threshold';
threshold.name = 'Threshold for binary lesion map';
threshold.strtype = 'e';
threshold.num = [1 1];
threshold.def  = @(val) 0; 
threshold.help = {[...
'The threshold for computing binary lesion maps. This value should be '...
'a number in the interval (0,1].']};

segopts = cfg_branch;
segopts.tag = 'segopts';
segopts.name = 'Options for lesion segmentation';
segopts.val = {initial,belief,mrf,maxiter,threshold};
segopts.help = {['Options for the lesion growth model.']};

% ------------------------------------------------------------------------

prob    = cfg_menu;
prob.tag = 'prob';
prob.name = 'Lesion probability map';
prob.labels = {'none','yes'};
prob.values = {0 1};
prob.def = @(val) 1;
prob.help    = {['The probability lesion map assigns a value in the range '...
    'of [0,1] to all voxels.']};

binary    = cfg_menu;
binary.tag = 'binary';
binary.name = 'Binary lesion map';
binary.labels = {'none','yes'};
binary.values = {0 1};
binary.def = @(val) 0;
binary.help    = {['The binary lesion map consists of voxels with either '...
    'a 0 or a 1, where the lesion probability for a voxel with a 1 is at'...
    'least as high as the threshold for the binary lesion map.']};

normalized    = cfg_menu;
normalized.tag = 'normalized';
normalized.name = 'Normalized lesion map';
normalized.labels = {'none','yes'};
normalized.values = {0 1};
normalized.def = @(val) 0; 
normalized.help    = {['Choose wheather the lesion probaility map and/or '...
    'the binary lesion map should be normalized. The forward deformation '...
    'field that results from the PVE-label estimation is used for normalization.']};

lesions      = cfg_branch;
lesions.tag = 'lesions';
lesions.name = 'Lesion Map';
lesions.val = {prob,binary,normalized};
lesions.help     = {'Writing options for the segmented lesions.'};

other      = cfg_menu;
other.tag = 'other';
other.name = 'Other images';
other.labels = {'no','yes'};
other.values = {0 1};
other.def  = @(val) 1;
other.help = {['Per default, images that are needed for lesion segmentation are '...
    'saved. To be precise, these images are the T1 and FLAIR images '...
    'corrected for bias, the coregistered bias-corrected FLAIR image, '...
    'the estimated PVE label as well as the forward and inverse '...
    'deformation field. These images are needed to run the lesion growth '...
    'algorithm again (e.g. with a different initial threshold) without '...
    'estimating the PVE label and the bias correction of the FLAIR image.']};

output      = cfg_branch;
output.tag = 'output';
output.name = 'Writing options';
output.val = {lesions,other};
output.help = {['']};

% ------------------------------------------------------------------------

segment = cfg_exbranch;
segment.tag = 'segment';
segment.name = 'LST: Segment MS lesions by lesion growing';
segment.val = {data_pve,data_FLAIR,segopts,output};
segment.prog   = @ps_LST_run;
segment.vout = @vout;
segment.help   = {['This tool implements a lesion growth algorithm for '...
    'the segmentation of lesions in Multiple Sclerosis (MS).'...
    ''...
    'THe algorithm is explained in detail in Schmidt et al. (2012). See '...
    'the manual for the usage of this tool.']};

%% Determination of the initial threshold ---------------------------------

data_plm = cfg_files;
data_plm.tag  = 'data_plm';
data_plm.name = 'Probability lesion maps';
data_plm.help = {[...
'Select probability lesion maps which are to be compared with the reference '...
'segmentations. The number of chosen lesion probabilty maps for one '...
'subject is of lesser importance than the fact that for every subject one '...
'reference image is present in the same folder as the probablity lesion maps.']};
data_plm.filter = 'image';
data_plm.ufilter = '.*';
data_plm.num     = [1 Inf];

data_ref = cfg_files;
data_ref.tag  = 'data_ref';
data_ref.name = 'Reference Segmentation';
data_ref.help = {[...
'Select binary reference segmentations. These images needed to be in the '...
'same folder as the correspondent probabilty lesion maps']};
data_ref.filter = 'image';
data_ref.ufilter = '.*';
data_ref.num     = [1 Inf];

bin_thresh = cfg_entry;
bin_thresh.tag = 'bin_thresh';
bin_thresh.name = 'Threshold for binary lesion maps';
bin_thresh.strtype = 'e';
bin_thresh.num = [1 1];
bin_thresh.def  = @(val) 1; 
bin_thresh.help = {[...
'The threshold for computing binary lesion maps that are to be compared '...
'with the reference segmentations. This value should be a number in the '...
'interval (0,1].']};

doit = cfg_exbranch;
doit.tag = 'doit';
doit.name = 'Determination of the optimal initial threshold';
doit.val = {data_plm,data_ref,bin_thresh};
doit.prog = @ps_LST_doit;
doit.help = {['With this module we offer the opportunity to determine '...
    ' the optimal κ based on the Dice coefficient, see Schmidt et al. (2012) '...
    'for details. This requires the existence of a reference segmentation '...
    'to be compared with the lesion map. This reference image is a binary '...
    'image in the space of the T1 image where a 1 indicates a lesion.']};


%% Lesion filling --------------------------------------------------------

data_T1 = cfg_files;
data_T1.tag  = 'data_T1';
data_T1.name = 'T1 volumes';
data_T1.help = {[...
'Select original T1 images.']};
data_T1.filter = 'image';
data_T1.ufilter = '.*';
data_T1.num     = [1 Inf];

data_lm = cfg_files;
data_lm.tag  = 'data_lm';
data_lm.name = 'Lesion maps';
data_lm.help = {[...
'Lesion maps, either the probability lesion map or the binary lesion map. '...
'The probabilty lesion map will give better results.']};
data_lm.filter = 'image';
data_lm.ufilter = '^lesion_.*';
data_lm.num     = [1 Inf];

native_filled    = cfg_menu;
native_filled.tag = 'native_filled';
native_filled.name = 'Native space';
native_filled.labels = {'none','yes'};
native_filled.values = {0 1};
native_filled.def = @(val) 1; 
native_filled.help    = {'The filled original T1 image in native space.'};

bias_filled    = cfg_menu;
bias_filled.tag = 'bias_filled';
bias_filled.name = 'Bias corrected';
bias_filled.labels = {'none','yes'};
bias_filled.values = {0 1};
bias_filled.def = @(val) 0; 
bias_filled.help    = {'Bias-corrected filled T1 image in native space.'};

normalized_filled    = cfg_menu;
normalized_filled.tag = 'normalized_filled';
normalized_filled.name = 'Normalized';
normalized_filled.labels = {'none','yes'};
normalized_filled.values = {0 1};
normalized_filled.def = @(val) 0;
normalized_filled.help    = {'Filled T1 images in MNI space. For this '...
    'option, the forward deformation field is required.'};

T1_filled      = cfg_branch;
T1_filled.tag = 'T1_filled';
T1_filled.name = 'T1 filled';
T1_filled.val = {native_filled,bias_filled,normalized_filled};
T1_filled.help     = {'Writing options for the filled T1 images.'};

output_lesfill      = cfg_branch;
output_lesfill.tag = 'output_lesfill';
output_lesfill.name = 'Writing options';
output_lesfill.val = {T1_filled};
output_lesfill.help = {['']};

lesfill = cfg_exbranch;
lesfill.tag = 'lesfill';
lesfill.name = 'Lesion filling';
lesfill.val = {data_T1,data_lm,output_lesfill};
lesfill.prog = @ps_LST_lesfill;
lesfill.vout = @vout_lesfill;
lesfill.help = {['This module can be used to fill the segmented lesions '...
    'in the T1 image with estimated healthy white matter tissue. The '...
    'underlying algorithm is inspired by a method proposed by Chard et al. (2010).']};

%% -----------------------------------------------------------------------

data_plm_thresh = cfg_files;
data_plm_thresh.tag  = 'data_plm_thresh';
data_plm_thresh.name = 'Probability lesion maps';
data_plm_thresh.help = {[...
'Select probability lesion maps.']};
data_plm_thresh.filter = 'image';
data_plm_thresh.ufilter = '.*';
data_plm_thresh.num     = [1 Inf];

lesthresh = cfg_exbranch;
lesthresh.tag = 'lesthresh';
lesthresh.name = 'Lesion thresholding';
lesthresh.val = {data_plm_thresh,threshold};
lesthresh.prog = @ps_LST_thresholding;
lesthresh.vout = @vout_lesthresh;
lesthresh.help = {['This module can be used to threshold probability lesion maps to compute binary lesion maps.']};

%% -----------------------------------------------------------------------

adiof    = cfg_menu;
adiof.tag = 'adiof';
adiof.name = 'All data in one file';
adiof.labels = {'no','yes'};
adiof.values = {0 1};
adiof.def = @(val) 1; 
adiof.help    = {'If there is more than one image, should the values for the lesion'...
    'volume be written to one file? The data are written to a CSV file which is'...
    'written to the directory of the first image.'};

lesvolume = cfg_exbranch;
lesvolume.tag = 'lesvolume';
lesvolume.name = 'Total lesion volume';
lesvolume.val = {data_plm_thresh,adiof};
lesvolume.prog = @ps_LST_lesvolume;
lesvolume.help = {['This module computes the total lesion volume (ml) for given probability lesion maps.' ...
    'The user can either select probability lesion maps or binary lesion maps' ...
    'All voxels with values greater than zero will count as lesions, therefore the final total lesion volume will depend on this choice.' ...
    'The result is written to a .txt file.']};

%% -----------------------------------------------------------------------

lesiongrow = ps_LST_lesiongrowing;

%------------------------------------------------------------------------

LST  = cfg_choice;
LST.name = 'LST';
LST.tag  = 'LST';
LST.values = {segment,doit,lesfill,lesiongrow,lesthresh,lesvolume};

%------------------------------------------------------------------------
function dep = vout(job)

opts  = job.output;
cdep = cfg_dep;

if opts.lesions.prob
    cdep(end).sname      = 'Lesion probability map';
    cdep(end).src_output = substruct('.','lpm','()',{':'});
    cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    
    if opts.lesions.normalized
        cdep(end+1)          = cfg_dep; 
        cdep(end).sname      = 'Normalized lesion probability map';
        cdep(end).src_output = substruct('.','wlpm','()',{':'});
        cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    end
end

if opts.lesions.binary
    if opts.lesions.prob
        cdep(end+1) = cfg_dep; 
    end
    cdep(end).sname      = 'Binary lesion map';
    cdep(end).src_output = substruct('.','blm','()',{':'});
    cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    
    if opts.lesions.normalized
        cdep(end+1)          = cfg_dep; 
        cdep(end).sname      = 'Normalized binary lesion map';
        cdep(end).src_output = substruct('.','wblm','()',{':'});
        cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    end
end

dep = cdep;

function dep = vout_lesfill(job)

opts  = job.output_lesfill;
cdep = cfg_dep;

if opts.T1_filled.native_filled
    cdep(end).sname      = 'Filled T1 images in native space';
    cdep(end).src_output = substruct('.','T1_filled','()',{':'});
    cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    
    if opts.T1_filled.normalized_filled
        cdep(end+1)          = cfg_dep; 
        cdep(end).sname      = 'Normalized filled T1 images';
        cdep(end).src_output = substruct('.','wT1_filled','()',{':'});
        cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    end
end

if opts.T1_filled.bias_filled
    if opts.T1_filled.native_filled
        cdep(end+1) = cfg_dep; 
    end
    cdep(end).sname      = 'Bias-corrected filled T1 images in native space';
    cdep(end).src_output = substruct('.','mT1_filled','()',{':'});
    cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    
    if opts.T1_filled.normalized_filled
        cdep(end+1)          = cfg_dep; 
        cdep(end).sname      = 'Normalized bias-corrected filled T1 images';
        cdep(end).src_output = substruct('.','wmT1_filled','()',{':'});
        cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
    end
end

dep = cdep;

function dep = vout_lesthresh(job)

cdep = cfg_dep;

cdep(end).sname      = 'Binary lesion map';
cdep(end).src_output = substruct('.','blm','()',{':'});
cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});

dep = cdep;